; Delay.asm 20200517 Demo library module in ASM for Nuvoton N76E003 (SDCC version)
; Use sdas8051.exe and sdcc.exe version 4.0.0 inside Eclipse with auto make.
;
; Use this template for ASM programming with SDAS in SDCC in Exlipse IDE.
; You probably must set or #define FOSC_160000 before it will compile!
;
; To assemble and link this file use Eclipse auto-make or invoke the following commands:
;	sdas8051.exe -losffgp startup.S
;	sdcc.exe <options> startup.rel
; Defining custom area names in DATA or BIT will collide with similar hard-coded areas!
;-----------------------------------------------------------------------------------------------
; Included header files and libraries which are needed by this module go here.
.include "N76E003.asm"		; register and bit definition file for Nuvoton N76E003
;-----------------------------------------------------------------------------------------------
; The name of this module and command line compiler or linker options go here.
.module Delay				; the name of this module goes here but this is optional
;	.optsdcc -mmcs51 --model-small	; SDCC compiler emits this so may not be needed
;-----------------------------------------------------------------------------------------------
; Export .GLOBAL symbols to other modules or alternately define them in the code sections below.
;	.GLOBL symbol_name		; example global symbol exportation.
;-----------------------------------------------------------------------------------------------
; Document .GLOBL symbols imported form other modules or do this in the code sections below.
;	.GLOBL symbol_name		; example global symbol documentation
;-----------------------------------------------------------------------------------------------
; Special function register and bit declarations go here if .include file was not used above.
;	symbol_name .EQU 0x9C	; example register or bit definition
;-----------------------------------------------------------------------------------------------
; Add constant aka typeless numbers here. These could be system configuration symbols etc.
;	symbol_name .EQU 0x00	; example definition
;-----------------------------------------------------------------------------------------------
.page						; force page break in listing, RAM areas continue below
;-----------------------------------------------------------------------------------------------
.AREA REG_BANK_0 (OVR,DATA) ; absolute overlayable space for register bank 0
	.DS		8
.AREA REG_BANK_1 (OVR,DATA) ; uncomment to reserve space for register bank 1
;	.DS		8
.AREA REG_BANK_2 (OVR,DATA) ; uncomment to reserve space for register bank 2
;	.DS		8
.AREA REG_BANK_3 (OVR,DATA) ; uncomment to reserve space for register bank 3
;	.DS		8
; the above fucntionality is hard coded so do not attempt to do anything else with it!
;-----------------------------------------------------------------------------------------------
.AREA RSEG (ABS,DATA)		; absolute global register space in DATA area 0x00 ~ 0x80
;	.ORG 0x10				; anchor to address of specific register or register set
;	symbol_name: .DS 8		; example register space reservation
;-----------------------------------------------------------------------------------------------
.AREA OSEG (OVR,DATA) 		; overlayable space in DATA area 0x00 ~ 0x80
;	symbol_name: .DS 2		; example space reservation
;-----------------------------------------------------------------------------------------------
.AREA SSEG (DATA)			; global MCU stack space in IDATA area 0x00 ~ 0xFF
;__start__stack:: .DS 1		; all residual space will be assigned despite reserving only 1 byte
; Do not attempt to define any additional symbols since they will just be silently ignored.
; You probably don't want to be allocating anything here in the case of a library module.
;-----------------------------------------------------------------------------------------------
.AREA ISEG (DATA)			; global space in IDATA area 0x00 ~ 0xFF
;	symbol_name: .DS 3		; example space reservation
;-----------------------------------------------------------------------------------------------
.AREA IABS (ABS,DATA)		; absolute global space in IDATA area 0x00 ~ 0xFF
;	.ORG 0xFE				; anchor to address of memory location or register
;	symbol_name: .DS 2		; example space reservation or register definition
;-----------------------------------------------------------------------------------------------
.AREA BSEG (BIT)			; global space in BIT area 0x00 ~ 0x7F shown in RAM layout as B
;	symbol_name: .DS 4		; example reserving 4 BITs which use up half a byte at DATA 0x20
; Corresponding DATA area range 0x20 ~ 0x2F is automatically reserved as bits are allocated.
; The ABS directive does not function in BIT areas so the reservation is just silently dropped!
; A possible workaround is to allocate the corresponding byte in DATA area range 0x20 ~ 0x2F.
;-----------------------------------------------------------------------------------------------
.AREA BSEG_BYTES (DATA)		; global bit adressable space in DATA area 0x20 ~ 0x2F shown as B
;	symbol_name: .DS 1		; example reserving 1 byte allocation containing 8 bit allocations
; Corresponding BIT area range 0x00 ~ 0x7F is automatically reserved as bytes are allocated.
;-----------------------------------------------------------------------------------------------
.AREA BIT_BANK (OVR,DATA)	; global bit adressable space in DATA area 0x20 ~ 0x2F shown as T
;	symbol_name: .DS 1		; example reserving 1 byte allocation containing 8 bit allocations
; These allocations are not associated with BSEG (BIT) or BSEG_BYTES (DATA) but are useful for
; re-entrant functions needing bit parameters or variables which are easy to push and pop.
;-----------------------------------------------------------------------------------------------
.AREA PSEG (PAG,XDATA)		; global space in PDATA inpage area 0x??00 ~ 0x??FF
;	symbol_name: .DS 160	; example space reservation with 256 byte page allignment
; The ABS directive does not function within PDATA inpage so allignment errors are returned!
;-----------------------------------------------------------------------------------------------
.AREA XSEG (XDATA)			; global space in XDATA area 0x0000 ~ 0xFFFF
;	symbol_name: .DS 384	; example space reservation
; If XDATA does not start at 0x0000 then customize it using linker flag: --xram-loc 0x????
;-----------------------------------------------------------------------------------------------
.AREA XABS (ABS,XDATA)		; absolute global space in XDATA area 0x0000 ~ 0xFFFF
;	.ORG 0xD000				; anchor to address of memory location or I/O port
;	symbol_name: .DS 1		; example space reservation or port definition
;-----------------------------------------------------------------------------------------------
	.AREA XISEG (XDATA)		; global space in initialized XDATA area 0x0000 ~ 0xFFFF
;	symbol_name: .DS 300		; example space reservation
; Use this with NVRAM or if initialization code copies values from CODE to XDATA at startup.
;-----------------------------------------------------------------------------------------------
.page						; force page break in listing, ROM code areas continue below
;-----------------------------------------------------------------------------------------------
.AREA HOME (ABS,CODE) ; for reset and interrupt vectors at 0x0000 as defined in hardware
	.ORG 0x0000					; reset vector should only be defined in main
	.ORG 0x0003					; external interrupt 0
	.ORG 0x000B					; timer 0 overflow
	.ORG 0x0013					; external interrupt 1
	.ORG 0x001B					; timer 1 overflow
	.ORG 0x0023					; serial port 0 interrupt
	.ORG 0x002B					; timer 2 event
	.ORG 0x0033					; I2C status/timer-out interrupt
	.ORG 0x003B					; pin interrupt
	.ORG 0x0043					; brown-out detection interrupt
	.ORG 0x004B					; SPI interrupt
	.ORG 0x0053					; WDT interrupt
	.ORG 0x005B					; ADC interrupt
	.ORG 0x0063					; input capture interrupt
	.ORG 0x006B					; PWM interrupt
	.ORG 0x0073					; fault brake interrupt
	.ORG 0x007B					; serial port 1 interrupt
	.ORG 0x0083					; timer 3 overflow
	.ORG 0x008B					; self wake-up timer interrupt
;-----------------------------------------------------------------------------------------------
.AREA CSEG2 (CODE) ; our example subroutines in this external library module

; function void Timer0_Delay100us(UINT32 u32CNT)
; u32CNT allocated to registers r4 r5 r6 r7
.globl _Timer0_Delay100us
_Timer0_Delay100us:
 	mov	r4,dpl
 	mov	r5,dph
 	mov	r6,b
 	mov	r7,a
	anl	CKCON,#0xf7			; T0M=0, Timer0 Clock = Fsys/12
 	orl	TMOD,#0x01			; Timer0 is 16-bit mode
 	setb	TR0				; Start Timer0
00104$:
 	mov	a,r4
 	orl	a,r5
 	orl	a,r6
 	orl	a,r7
 	jz	00106$				; while (u32CNT != 0)
 	mov	TL0,#0x7e			; TL0 = LOBYTE(TIMER_DIV12_VALUE_100us)
 	mov	TH0,#0xff			; TH0 = HIBYTE(TIMER_DIV12_VALUE_100us)
00101$:
	jbc	TF0,00127$
	sjmp	00101$			; while (TF0 != 1)
00127$:
	dec	r4
	cjne	r4,#0xff,00128$
	dec	r5
	cjne	r5,#0xff,00128$
	dec	r6
	cjne	r6,#0xff,00128$
	dec	r7
00128$:
	sjmp	00104$
00106$:
	clr	TR0					; Stop Timer0
	ret

; function void _Timer0_Delay1ms(UINT32 u32CNT)
; u32CNT allocated to registers r4 r5 r6 r7
.globl _Timer0_Delay1ms
_Timer0_Delay1ms:
	mov	r4,dpl
	mov	r5,dph
	mov	r6,b
	mov	r7,a
	anl	CKCON,#0xf7			; T0M=0, Timer0 Clock = Fsys/12
	orl	TMOD,#0x01			; Timer0 is 16-bit mode
	setb	TR0				; Start Timer0
00104$:
	mov	a,r4
	orl	a,r5
	orl	a,r6
	orl	a,r7
	jz	00106$				; while (u32CNT != 0)
	mov	TL0,#0xca			; TL0 = LOBYTE(TIMER_DIV12_VALUE_1ms)
	mov	TH0,#0xfa			; TH0 = HIBYTE(TIMER_DIV12_VALUE_1ms)
00101$:
	jbc	TF0,00127$
	sjmp	00101$			; while (TF0 != 1)
00127$:
	dec	r4
	cjne	r4,#0xff,00128$
	dec	r5
	cjne	r5,#0xff,00128$
	dec	r6
	cjne	r6,#0xff,00128$
	dec	r7
00128$:
	sjmp	00104$
00106$:
	clr	TR0					; Stop Timer0
	ret

; function void Timer1_Delay10ms(UINT32 u32CNT)
; u32CNT allocated to registers r4 r5 r6 r7
.globl _Timer1_Delay10ms
_Timer1_Delay10ms:
	mov	r4,dpl
	mov	r5,dph
	mov	r6,b
	mov	r7,a
	anl	CKCON,#0xef			; T1M=0, Timer1 Clock = Fsys/12
	orl	TMOD,#0x10			; Timer1 is 16-bit mode
	setb	TR1				; Start Timer1
00104$:
	mov	a,r4
	orl	a,r5
	orl	a,r6
	orl	a,r7
	jz	00106$				; while (u32CNT != 0)
	mov	TL1,#0xea			; TL1 = LOBYTE(TIMER_DIV12_VALUE_10ms)
	mov	TH1,#0xcb			; TH1 = HIBYTE(TIMER_DIV12_VALUE_10ms)
00101$:
	jbc	TF1,00127$
	sjmp	00101$			; while (TF1 != 1)
00127$:
	dec	r4
	cjne	r4,#0xff,00128$
	dec	r5
	cjne	r5,#0xff,00128$
	dec	r6
	cjne	r6,#0xff,00128$
	dec	r7
00128$:
	sjmp	00104$
00106$:
	clr	TR1					; Stop Timer1
	ret

; function void Timer2_Delay500us(UINT32 u32CNT)
; u32CNT allocated to registers r4 r5 r6 r7
.globl _Timer2_Delay500us
_Timer2_Delay500us:
	mov	r4,dpl
	mov	r5,dph
	mov	r6,b
	mov	r7,a
	anl	T2MOD,#0xbf			; clr_T2DIV2 Timer2 Clock = Fsys/4
	anl	T2MOD,#0xdf			; clr_T2DIV1
	orl	T2MOD,#0x10			; set_T2DIV0
	setb	TR2				; Start Timer2
00104$:
	mov	a,r4
	orl	a,r5
	orl	a,r6
	orl	a,r7
	jz	00106$				; while (u32CNT != 0)
	mov	TL2,#0x30			; TL2 = LOBYTE(TIMER_DIV4_VALUE_500us)
	mov	TH2,#0xf8			; TH2 = HIBYTE(TIMER_DIV4_VALUE_500us)
00101$:
	jbc	TF2,00127$
	sjmp	00101$			; while (TF2 != 1)
00127$:
	dec	r4
	cjne	r4,#0xff,00128$
	dec	r5
	cjne	r5,#0xff,00128$
	dec	r6
	cjne	r6,#0xff,00128$
	dec	r7
00128$:
	sjmp	00104$
00106$:
	clr	TR2					; Stop Timer2
	ret

; function void Timer3_Delay100ms(UINT32 u32CNT)
; u32CNT allocated to registers r4 r5 r6 r7
.globl _Timer3_Delay100ms
_Timer3_Delay100ms:
	mov	r4,dpl
	mov	r5,dph
	mov	r6,b
	mov	r7,a
	mov	T3CON,#0x07			; Timer3 Clock = Fsys/128
	orl	T3CON,#0x08			; set_TR3 Trigger Timer3
00104$:
	mov	a,r4
	orl	a,r5
	orl	a,r6
	orl	a,r7
	jz	00106$				; while (u32CNT != 0)
	mov	RL3,#0x2c			; RL3 = LOBYTE(TIMER_DIV128_VALUE_100ms)
	mov	RH3,#0xcf			; RH3 = HIBYTE(TIMER_DIV128_VALUE_100ms)
00101$:
	mov	r2,T3CON
	anl	ar2,#0x10
	mov	r3,#0x00
	cjne	r2,#0x10,00101$
	cjne	r3,#0x00,00101$	; while ((T3CON&SET_BIT4) != SET_BIT4)
	anl	T3CON,#0xef			; clr_TF3
	dec	r4
	cjne	r4,#0xff,00129$
	dec	r5
	cjne	r5,#0xff,00129$
	dec	r6
	cjne	r6,#0xff,00129$
	dec	r7
00129$:
	sjmp	00104$
00106$:
	anl	T3CON,#0xf7			; clr_TR3 Stop Timer3
	ret

; function void Timer3_Delay10us(UINT32 u32CNT)
; u32CNT allocated to registers r4 r5 r6 r7
.globl _Timer3_Delay10us
_Timer3_Delay10us:
	mov	r4,dpl
	mov	r5,dph
	mov	r6,b
	mov	r7,a
	mov	T3CON,#0x07			; Timer3 Clock = Fsys/128
	orl	T3CON,#0x08			; set_TR3 Trigger Timer3
00104$:
	mov	a,r4
	orl	a,r5
	orl	a,r6
	orl	a,r7
	jz	00106$				; while (u32CNT != 0)
	mov	RL3,#0xd8			; RL3 = LOBYTE(TIMER_DIV4_VALUE_10us)
	mov	RH3,#0xff			; RH3 = HIBYTE(TIMER_DIV4_VALUE_10us)
00101$:
	mov	r2,T3CON
	anl	ar2,#0x10
	mov	r3,#0x00
	cjne	r2,#0x10,00101$
	cjne	r3,#0x00,00101$	; while ((T3CON&SET_BIT4) != SET_BIT4)
	anl	T3CON,#0xef			; clr_TF3
	dec	r4
	cjne	r4,#0xff,00129$
	dec	r5
	cjne	r5,#0xff,00129$
	dec	r6
	cjne	r6,#0xff,00129$
	dec	r7
00129$:
	sjmp	00104$
00106$:
	anl	T3CON,#0xf7			; clr_TR3 Stop Timer3
	ret
;-----------------------------------------------------------------------------------------------
.AREA CONST (CODE)				; space for CONSTants that need to be in ROM go here
;	symbol_name: .strz "Hi\n"	; example byte data elements in ROM
;-----------------------------------------------------------------------------------------------
.AREA XINIT (CODE)				; source ROM data for XISEG INITialization
;	symbol_name: .DB 0x82,0x90	; example byte data elements in ROM
;-----------------------------------------------------------------------------------------------
.AREA CABS (ABS,CODE)			; Code ABSolute may be an input port or contain ROM data
;	.ORG 0xC000					; anchor to address of input port or ROM data
;	symbol_name: .DS 1			; example input port definition
;	symbol_name: .DB 0x67,0x55	; example byte data elements in ROM
;-----------------------------------------------------------------------------------------------
.page							; force page break in listing
;-----------------------------------------------------------------------------------------------
;END							; end of file marker
